//
//  STAppDelegate.m
//  Safe Terminal
//
//  Created by Nir on 22/02/06.
//  Copyright 2006 Nir Soffer. All rights reserved.
//

#import "STAppDelegate.h"


@implementation STAppDelegate

- (id)init;
{
    /* Check if we should install and observe
    applicationDidFinishLounching in which we will try to install the
    delegate */
    
    if ((self = [super init])) {
        if (! [self install]) {
            [self dealloc];
            return nil;
        }
    }
    return self;    
}

- (BOOL)install;
{    
    /* Set me as Terminal.app application delegate, and the original
    delegate as my delegate. This allow overiding or extending original
    delegate methods. */
    
    NSString *bundleIdentifier = [[NSBundle mainBundle] bundleIdentifier];
    if (! [bundleIdentifier isEqualToString:@"com.apple.Terminal"])
        return NO;
    
    [self setDelegate:[NSApp delegate]];
    [NSApp setDelegate:self];
    NSLog(@"Safe Terminal loaded.");
    
    [[NSNotificationCenter defaultCenter]
        addObserver:self
           selector:@selector(applicationDidFinishLounching:)
               name:NSApplicationDidFinishLaunchingNotification
             object:NSApp];        
    return YES;
}

- (void)applicationDidFinishLounching:(NSNotification *)notification;
{    
    /* Remind the application to open a new shell. I don't know why it
    will not open one without this call. */

    [[NSNotificationCenter defaultCenter] removeObserver:self];
    [NSApp sendAction:@selector(new:) to:nil from:self];
}

- (BOOL)application:(NSApplication *)theApplication openFile:(NSString *)filename
{
    /* Extend the original method with confirmation dialog */

    int returnCode = [[self alertForFilename:filename] runModal];
    if ((returnCode == NSAlertSecondButtonReturn) && 
        ([delegate respondsToSelector:@selector(application:openFile:)]))
        return [delegate application:theApplication openFile:filename];
    else
        return NO;
}

- (NSAlert *)alertForFilename:(NSString *)filename;
{
    /* Create an alert for filename 
    
    TODO: check the dialog with English speaking users.
    */

    NSAlert *alert = [[NSAlert alloc] init];
    [alert setAlertStyle:NSCriticalAlertStyle];
    [alert setMessageText:[NSString stringWithFormat:
        NSLocalizedString(@"Are you sure you want to execute %@?", nil),
        [[NSFileManager defaultManager] displayNameAtPath:filename]]];
    [alert setInformativeText:
        NSLocalizedString(@"Executing scripts might harm your computer.", 
                          nil)];
    [alert addButtonWithTitle:NSLocalizedString(@"Cancel", nil)];
    [alert addButtonWithTitle:NSLocalizedString(@"Execute Script", nil)];    
    return [alert autorelease];
}

- (void)setDelegate:(id)anObject;
{
    /* By convention Cocoa objects do not retain their delegates. */    
    delegate = anObject;
}

/* Delegate implementaion */

- (void)forwardInvocation:(NSInvocation *)invocation
{
    /* Forward anything I don't implement to the delegate */
    
    SEL aSelector = [invocation selector];
    
    if ([delegate respondsToSelector:aSelector])
        [invocation invokeWithTarget:delegate];
    else
        [self doesNotRecognizeSelector:aSelector];
}

- (NSMethodSignature *)methodSignatureForSelector:(SEL)aSelector
{
    /* Rquired to implement a delegate */
    
    return [delegate methodSignatureForSelector:aSelector];
}

@end
